
#include <string>
#include <fstream>
#include <vdr/menu.h>

#include <vdr/osd.h>
#include <vdr/menuitems.h>
#include <vdr/remote.h>

#include "sportngosd.h"
#include "sportngtools.h"
#include "setup.h"
#include "i18n.h"
#if 0
static const char* translate[]= {trNOOP("Test tr-String"),trNOOP("Test tr-String2")}
#endif
extern const char *SportNG_VERSION;



static char** SplitString(char* String, char Char, char* Strings[], int Count)
{
	Strings[0] = String;
	for(int i=1; i<Count; i++)
	{
		char *p = strchr(String, Char);
		if (p)
		{
			*p++ = 0;
			Strings[i] = String = p;
		}
		else
			Strings[i] = 0;
	}
	return Strings;
}
/* ########################################################
   ------------------- Men Refresh------------------------
   ######################################################## */

class cSportNGRefresh : public cOsdMenu
{
public:
	cSportNGRefresh(int Mode, const void* Var=NULL);
	enum {REFRESH, UPDATE};
private:
	eOSState Reload();
	eOSState Update(const char* File);
	void Completed(const char *Message);
	bool ExecCommand(const char *Message, const char *Command, bool NiceLine=true);
	eOSState ProcessKey(eKeys Key);
};

/* ########################################################
   ------------------- HelpKeys ---------------------------
   ######################################################## */

cSportNGHelpKeys::cSportNGHelpKeys()
{
	for(int i=0;i<4;i++)	helpKeys[i]=helpCommands[i]=NULL;
}

//##################################################################

cSportNGHelpKeys::~cSportNGHelpKeys()
{
	ClearHelpKeys();
}

//##################################################################

void cSportNGHelpKeys::SetHelpKeyAndCommand(int Idx, const char *HelpKey, const char* Command)
{
	if(0<=Idx && Idx<=3)
	{
		free(helpKeys[Idx]);
		helpKeys[Idx] = HelpKey ? strdup(HelpKey) : NULL;
		free(helpCommands[Idx]);
		helpCommands[Idx] = Command ? strdup(Command) : NULL;
	}
}

//##################################################################

void cSportNGHelpKeys::SetHelpKeyAndCommand(int Idx, char *KeyAndCommand)
{
	if(0<=Idx && Idx<=3 && KeyAndCommand)
	{
		char *c=0;
		if(KeyAndCommand[0]=='*' && KeyAndCommand[1]==0)
			c = KeyAndCommand;
		else if(KeyAndCommand && (c=strchr(KeyAndCommand, '=')))
			*c++=0;
		if(c)
		{
			SetHelpKeyAndCommand(Idx, KeyAndCommand, c);
		}
	}
}

//##################################################################

void	cSportNGHelpKeys::ClearHelpKeys()
{
	for(int i=0; i<4;i++)	// HelpKeys Clear
	{
		free(helpKeys[i]); helpKeys[i]=NULL;
		free(helpCommands[i]); helpCommands[i]=NULL;
	}
}

//##################################################################

const char *cSportNGHelpKeys::GetHelpCommand(int Idx)	// 0=red, 1=green, 2=yellow, 3=blue 
{
	if(0<=Idx && Idx<=3)
		if(helpKeys[Idx])
			return helpCommands[Idx];
	return NULL;
}

//##################################################################

char **cSportNGHelpKeys::GetHelpKeys(char *HelpKeys[], bool CopyAll=true)
{
	for(int i=0;i<4;i++)
	{
		if(helpKeys[i] || CopyAll)
		{
			if(helpKeys[i] && helpKeys[i][0]=='*')
				HelpKeys[i] = NULL;
			else
				HelpKeys[i] = helpKeys[i];
		}
	}
	return HelpKeys;	
}

/* ########################################################
   ------------------- cSportNGBase -----------------------
   ######################################################## */
int cSportNGBase::running=0;

cSportNGBase::cSportNGBase()
{
	pFileName=0;
	FileTime=0;
	running++;
};

//##################################################################

cSportNGBase::~cSportNGBase()
{
	running--;
	free(pFileName);
};

//##################################################################

bool cSportNGBase::LoadContent(const char *File)
{
	if(Running() && SportNGSetup.s_update==2) // automatisches Update
		if(cOsdMenu *SportNGRefresh = cSportNG::reloadData(false))
			AddSubMenu(SportNGRefresh);
	bool reLoad = true;
	if(pFileName != File)
	{
		reLoad = false;
		free(pFileName);
		if(File)
			pFileName = strdup(File);
		else
			pFileName = 0;
	}
	bool ret=true;
	if(pFileName)
	{
		ret=false;
		cString FullFileName = cString::sprintf("%s/%s", DataDir, pFileName);
		struct stat buf;
		time_t newFileTime = stat(FullFileName,&buf)?0: buf.st_mtime;
		if(reLoad && FileTime == newFileTime)	// wenn die selbe Datei geladen werden Soll 
			return true;								// aber das Datum gleich ist ---> nichts tun 
		FileTime = newFileTime;


		unsigned int filesize=0;
		FILE *f = fopen(FullFileName, "r");
		if(f)
		{
			isyslog("%s: loading '%s'", PLUGIN_NAME_I18N, pFileName);
			fseek(f,0,SEEK_END);
			filesize= ftell(f);
			fseek(f,0,SEEK_SET);
			if(char* buffer = MALLOC(char, filesize+1))
			{
				if(fread(buffer, 1, filesize, f)==filesize)
				{
					buffer[filesize] = 0;
					SetContent(buffer, /*i18n=*/true , /*fromLoad=*/true );
					ret=true;
				}
				free(buffer);
			}
			fclose(f);
		}
		else
			esyslog("%s: Can't open '%s'\n",PLUGIN_NAME_I18N, pFileName);
	}
	return ret;
}

//##################################################################

extern const char	SportNGInfo[];
eOSState cSportNGBase::ParseCommand(const char* Command)
{
	if(Command)
	{
		switch(*Command)
		{
			case 'b':	// Back
				return osBack;
				break;
			case 'i':	// Info
//				return AddSubMenu (new cSportNGInfo);
//				class cSportNGMenuText;
				return AddSubMenu (new cSportNGMenuText(SportNGInfo, true));
//				return AddSubMenu (new cSportNGMenuText("SportNG@Fussball@1. Bundesliga@2008-2009@Bayern Muenchen-Arminia Bielefeld"));
			case 'l':	// Load new Content
	isyslog("%s: load-command %s", PLUGIN_NAME_I18N, &Command[1]);
				LoadContent(&Command[1]);	
				break;
			case 'r':	// Download new Data
				if(cOsdMenu *SportNGRefresh = cSportNG::reloadData())
					AddSubMenu(SportNGRefresh);
/*				if(cOsdMenu cSportNG::reloadData())
				{
					cRemote::CallPlugin(PLUGIN_NAME_I18N);
					return osEnd;
				}
*/
				break;
			case 's':	// open SubMen
				return AddSubMenu(new cSportNG(&Command[1]));
			case 't':	// open TextSubMen
				return AddSubMenu(new cSportNGMenuText(&Command[1]));
			case 'u':	// Update
				return AddSubMenu(new cSportNGRefresh(cSportNGRefresh::UPDATE, &Command[1]));
			case 'x':	// Exit
				return osEnd;
				break;
			default:
				;	
		}
	}
	return osContinue;
}

/* ########################################################
   ------------------- SportNG Item -----------------------
   ######################################################## */

class cSportNGItem : public cSportNGHelpKeys, public cOsdItem 
{
private:
	char *command;
public:
	cSportNGItem(char *Datas[]) : cOsdItem(Datas[1], osUnknown, true)
	{
		command=Datas[2] ? strdup(Datas[2]) : NULL;
		for(int i=0;i<4;i++)
			SetHelpKeyAndCommand(i, Datas[i+3]);
	}
	cSportNGItem(const char *Text) : cOsdItem(Text, osUnknown, false)
	{
		command = NULL;
	}
	~cSportNGItem()
	{
		if(command)	free(command);
	}
	const char* GetCommand(void)
	{
		return command;
	}
};

/* ########################################################
   -------------- H A U P T M E N  -----------------------
   ######################################################## */

const char *EM[] = {
			trNOOP("Error while loading"),
			trNOOP("Error while loading"),
			trNOOP("File:%s"),
			trNOOP("Press 'Yellow' to refresh the data."),
			trNOOP("Check your systemlog for errormessages."),
			trNOOP("Turn on 'Autoupdate' (not recommened)."),
			trNOOP("Try other paths with commandline flags"),
			trNOOP("If nothing helps, get an internetconnection.")
};


//class cSportNGRefresh;
cSportNG::cSportNG(const char *File)
:cOsdMenu("")
{
	if(!File) return;
	lastItem = -1;
	Display(); // !!! WICHTIG damit DisplayMenu()->GetTextAreaWidth() korrekten Wert liefert
	isyslog("%s: Load Menu from '%s'\n",PLUGIN_NAME_I18N, File);
	if(LoadContent(File))
	{
		isyslog("%s: Loaded Menu from '%s'\n",PLUGIN_NAME_I18N, File);
	}
	else
	{
		esyslog("%s: Can't load Menu from '%s'\n",PLUGIN_NAME_I18N, File);
		SetContent(cString::sprintf("t|%s\nl|%s\nm|%s|r\nl|%s\nl|%s\nl|%s\nl|%s\nl|%s",
								tr(EM[0]), tr(EM[1]), (const char*)cString::sprintf(tr(EM[2]), File), tr(EM[3]), tr(EM[4]), tr(EM[5]), tr(EM[6]), tr(EM[7])
						), /*i18n=*/false , /*fromLoad=*/true );
	}
	Display();
	DisplayHelpKeys();
	
	if(Running()==1 && SportNGSetup.s_update==1) // automatisches Update
		if(cOsdMenu *SportNGRefresh = cSportNG::reloadData(false))
			AddSubMenu(SportNGRefresh);
}
//##################################################################
/*
cSportNG::~cSportNG()
{
}
*/

//##################################################################
int cSportNG::GetTextAreaWidth()
{
	const cFont *pFont = DisplayMenu()->GetTextAreaFont(false);
	if (!pFont) pFont = cFont::GetFont(fontOsd);
	return DisplayMenu()->GetTextAreaWidth() - (3-SportNGSetup.s_adjust1)*pFont->Width(' ');
}
//###########################################################
/*
#define AverageCharacterWidth	(max(14-SportNGSetup.s_adjust2,5))
int cSportNG::GetAverageCharacterWidth(void)
{
	const cFont *pFont = DisplayMenu()->GetTextAreaFont(false);
	if (!pFont) pFont = cFont::GetFont(fontOsd);
	return ::GetAverageCharacterWidth(pFont, SportNGSetup.s_adjust2);
}
*/
//###########################################################

void cSportNG::CalcColWidth(const char *Text, int ColWidth[6], int LastColWidth[6])
{
	if(char *text = strdup(Text))
	{
		const cFont *pFont = DisplayMenu()->GetTextAreaFont(false);
		if (!pFont) pFont = cFont::GetFont(fontOsd);
		char *s = text;
		for(int i=0; i<6; i++)
		{
			char *e = strchrnul(s, '\t');
			if(*e)
			{
				*e++=0;
				int width = pFont->Width(s);
				if(ColWidth[i]<width)
					ColWidth[i]=width;
				 s=e;
			}
			else
			{
				if(*s)
				{
					int width = pFont->Width(s);
					if(LastColWidth[i]<width)
						LastColWidth[i]=width;
				}
				break;
			}
		}
		free(text);
	}
}

//##################################################################

void cSportNG::SetContent(const char *Text, bool i18n/*=true*/, bool fromLoad/*=false*/)
{
	if(!fromLoad)
		LoadContent(NULL);

	Clear();
	Skins.Flush();
	ClearHelpKeys();
	SetHelpKeyAndCommand(2, tr("Refresh"),"r");	// HelpKeys Std
	SetHelpKeyAndCommand(3, tr("Info"), "i");
	if(Text)
	{
#if VDRVERSNUM >= 10503
		cCharSetConv myCharSetConv("ISO-8859-15", cCharSetConv::SystemCharacterTable());
		#define CharConv(s) (char*)(i18n?myCharSetConv.Convert(s):s)
#else
		#define CharConv(s) s
#endif
		int ColWidth[6]		= {0, 0, 0, 0, 0, 0};
		char ColFlags[6]		= {'*','*','*','*','*','*'};
		int LastColWidth[6]	= {0, 0, 0, 0, 0, 0};
//		char *text = strdup(Text);
		char *s, *n, *text = n = strdup(Text);
		int line = 0;
		SetHasHotkeys();

//		while ((s = ReadLine.Read(f)) != NULL)
		while(n)
		{
			s = n;
			n = strchr(s, '\n');
			if(n) *n++ = 0;
			
			line++;
			if (!isempty(s))
			{
				if(s[0] && s[1]=='|')
				{
					s = CharConv(s);
					// m|MenText|OK-Command|redKey=redKeyCommand|greenKey=greenKeyCommand|yellowKey=yellowKeyCommand|blueKey=blueKeyCommand
					char *datas[7];

					SplitString(s, '|', datas, sizeof(datas)/sizeof(datas[0]));
					switch(datas[0][0])
					{
					case 'c':	// Col-Flags '+'=darf vergrert werden '-'=darf verkleinert werden '*'=darf vergrert und verkleinert werden
						{
							int i;
							for(i=0; i<6; i++)
								ColFlags[i] = datas[i+1]&&datas[i+1][0] ? datas[i+1][0] : '*';
						}
						break;
					case 'h':	// Help-Keys setzen
						{
							for(int i=0;i<4;i++)	SetHelpKeyAndCommand(i, datas[i+1]);
						}
						break;
					case 'l':	// normale Zeile
						CalcColWidth(datas[1], ColWidth, LastColWidth);
						Add(new cSportNGItem(datas[1]));
						break;
					case 'm':	// Men
						CalcColWidth(datas[1], ColWidth, LastColWidth);
						Add(new cSportNGItem(datas));
						break;
					case 't':	// Titel setzen
						SetTitle(datas[1]);
						break;
					default:
						;
					}
				}
			}
		}
		free(text);
		const cFont *pFont = DisplayMenu()->GetTextAreaFont(false);
		if (!pFont) pFont = cFont::GetFont(fontOsd);
		int AreaWidth = GetTextAreaWidth()/AverageCharacterWidth;	// Gesamtbreite in AverageChars (abgerundet)
		int ColWidthSum=0;

#if 0
Add(new cSportNGItem(cString::sprintf("TextAreaWidth=%i(%i) ",DisplayMenu()->GetTextAreaWidth(), AreaWidth)));
Add(new cSportNGItem(cString::sprintf("AverageCharacterWidth=%i",AverageCharacterWidth)));
Add(new cSportNGItem(cString::sprintf("ColWidth=%i,%i,%i,%i,%i,%i ", ColWidth[0], ColWidth[1], ColWidth[2], ColWidth[3], ColWidth[4], ColWidth[5])));
Add(new cSportNGItem(cString::sprintf("LastColWidth=%i,%i,%i,%i,%i,%i ", LastColWidth[0], LastColWidth[1], LastColWidth[2], LastColWidth[3], LastColWidth[4], LastColWidth[5])));
#endif
		int i;
		for(i=0; i<6; i++)
			if(ColWidth[i] == 0) ColWidth[i] = LastColWidth[i];
		for(i=0; i<6 && ColWidth[i]; i++)
		{
			ColWidth[i] = (ColWidth[i]+AverageCharacterWidth-1)/AverageCharacterWidth;	// Breite in AverageChars (aufgerundet)
			ColWidthSum += ColWidth[i];
		}
		if(ColWidthSum != AreaWidth) // Spalten mssen vergrert/verkleinert werden
		{
			char cmp;
			if(ColWidthSum < AreaWidth)
				cmp='+'; // Spalten mssen vergrert werden
			else
				cmp='-'; // Spalten mssen verkleinert werden
						
			for(int i=0; i<6 && ColWidth[i]; i++)
			{
				if(ColFlags[i] != '+' && ColFlags[i] != '*')
				{
					AreaWidth -= ColWidth[i];			// Cols, die nicht vergrert/verkleinert werden drfen aus Summen entfernen
					ColWidthSum -= ColWidth[i];
				}
			}
			int Rest = 0;
			for(int i=0; i<6 && ColWidth[i]; i++)
			{
				if(ColFlags[i] == '+' || ColFlags[i] == '*')
				{
					ColWidth[i] = ((ColWidth[i]*AreaWidth<<8)/ColWidthSum)+128+Rest;			// Cols, die vergrert/verkleinert werden drfen anpassen
					Rest = (ColWidth[i]&0xff)-128;														// RundungsDifferenz mitnehmen
					ColWidth[i] >>= 8;
				}
			}
		}
		SetCols(ColWidth[0], ColWidth[1], ColWidth[2], ColWidth[3], ColWidth[4]);

#if 0
Add(new cSportNGItem(cString::sprintf("ColWidth=%i,%i,%i,%i,%i,%i ", ColWidth[0], ColWidth[1], ColWidth[2], ColWidth[3], ColWidth[4], ColWidth[5])));

{
char line[17]; line[16]=0;
for(int l=32; l<256; l+=16)
{
for(int s=0; s<16;s++)
{
	line[s] = l+s;
}
	Add(new cSportNGItem(CharConv(cString::sprintf("Line %i = %s", l, line))));
}}

char* dummy[]={"","dummyTXT",NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL};
Add(new cSportNGItem(dummy));
#endif 
	}
	Display();
	DisplayHelpKeys(true);
}

//########################################################

void cSportNG::DisplayHelpKeys(bool Force)
{
	if(lastItem != Current() || Force)
	{
		lastItem = Current();
		char *currenthelpKeys[4];
		GetHelpKeys(currenthelpKeys);
		cSportNGItem *item = (cSportNGItem*)Get(Current());
		if(item)	item->GetHelpKeys(currenthelpKeys, false);
		SetHelp(currenthelpKeys[0],currenthelpKeys[1],currenthelpKeys[2],currenthelpKeys[3]);
	}
}

//#############################################################

eOSState cSportNG::ProcessKey(eKeys Key)
{
	bool hadSubmenu = HasSubMenu();

	eOSState state = cOsdMenu::ProcessKey(Key);

	if(hadSubmenu && !HasSubMenu())
	{
		if(GetFileName()) LoadContent(GetFileName());
		return osContinue;
	}

	if(state==osContinue)
		DisplayHelpKeys();// HelpKeys am Kontext des aktuellen Items anpassen
	else if(state==osUnknown)
	{
		state = osContinue;
		const char *Command=NULL;
		cSportNGItem *item = (cSportNGItem*)Get(Current());
		switch (Key)
		{
		case kRed:
			if(item)	Command = item->GetHelpCommand(0);
			if(!Command)	Command = GetHelpCommand(0);
			if(Command)	return ParseCommand(Command);
			break;
		case kGreen:
			if(item)	Command = item->GetHelpCommand(1);
			if(!Command)	Command = GetHelpCommand(1);
			if(Command)	return ParseCommand(Command);
			break;
		case kYellow:
			if(item)	Command = item->GetHelpCommand(2);
			if(!Command)	Command = GetHelpCommand(2);
			if(Command)	return ParseCommand(Command);
			break;
		case kBlue:
			if(item)	Command = item->GetHelpCommand(3);
			if(!Command)	Command = GetHelpCommand(3);
			if(Command)	return ParseCommand(Command);
			break;
		case kOk:
			if(item)	Command = item->GetCommand();
			if(Command)	return ParseCommand(Command);
			break;
		default:
			;
		}
	}
	return state;
}
//###########################################################
cOsdMenu *cSportNG::reloadData(bool Force)
{
	if(!Force)
	{
		Force = true;
		cString f_next_update = cString::sprintf("%s/%s", DataDir, "SportNG@NextUpdate");
		if(access(f_next_update, F_OK) == 0)
		{
			isyslog("%s: loading %s", PLUGIN_NAME_I18N, (const char*)f_next_update);
			FILE *f = fopen(f_next_update, "r");
			if(f)
			{
				char nextUpdate[20];
				char actualTime[20];
				struct tm tm_r;
				if(fread(nextUpdate, 1, 19, f)==19)
				{
					nextUpdate[19]=0;
					time_t t = time(NULL);
					strftime(actualTime, sizeof(actualTime), "%Y-%m-%d %H:%M:%S", localtime_r(&t, &tm_r));
					Force = strcmp(actualTime, nextUpdate)>=0;
					isyslog("%s: Time=%s Update=%s -> reload %sforced", PLUGIN_NAME_I18N, actualTime, nextUpdate, Force?"":"not ");
				}
				fclose(f);
			}
		}
	}
	if(Force)
		return new cSportNGRefresh(cSportNGRefresh::REFRESH);
	return NULL;
}
#if 0
bool cSportNG::reloadData(bool Force)
{
	if(!Force)
	{
		cString f_next_update = cString::sprintf("%s/%s", DataDir, "SportNG@NextUpdate");
		if(access(f_next_update, F_OK) == 0)
		{
			isyslog("%s: loading %s", PLUGIN_NAME_I18N, (const char*)f_next_update);
			FILE *f = fopen(f_next_update, "r");
			if(f)
			{
				char nextUpdate[20];
				char actualTime[20];
				struct tm tm_r;
				if(fread(nextUpdate, 1, 19, f)==19)
				{
					nextUpdate[19]=0;
					time_t t = time(NULL);
					strftime(actualTime, sizeof(actualTime), "%Y-%m-%d %H:%M:%S", localtime_r(&t, &tm_r));
					Force = strcmp(actualTime, nextUpdate)>=0;
					isyslog("%s: Time=%s Update=%s -> reload %sforced", PLUGIN_NAME_I18N, actualTime, nextUpdate, Force?"":"not ");
				}
				else
					Force = true;
				fclose(f);
			}
		}
	}
	if(Force)
	{
		cString cmd;
		Skins.Message(mtStatus, tr("Checking Network..."));
		cmd = cString::sprintf("/bin/sh %s/%s network", ScriptDir, "sportng.sh");
		int res = system(cmd);
		isyslog("%s: system('%s')=%i;\n", PLUGIN_NAME_I18N, (const char*)cmd, res);
		if(res)
		{
			Skins.Message(mtError, tr("NO NETRORK"), 2);
			return false;
		}
		Skins.Message(mtStatus, tr("Downloading data..."));
		cmd = cString::sprintf("/bin/sh %s/%s download %s sportng=%s", ScriptDir, "sportng.sh", DataDir, SportNG_VERSION);
		res = system(cmd);
		isyslog("%s: system('%s')=%i;\n", PLUGIN_NAME_I18N, (const char*)cmd, res);
		if(res)
		{
			Skins.Message(mtError, tr("Error while downloading data"), 2);
			return false;
		}
		Skins.Message(mtStatus,"Alles o.K.");
//		sleep(1);
		return true;
	}
	return false;
}
#endif
/* ########################################################
   ------------------- cSportNGMenuText -------------------
   ######################################################## */

//##################################################################

cSportNGMenuText::cSportNGMenuText(const char *File, bool isText) : cMenuText("", NULL)
{
	if(isText)
		SetContent(File);
	else if(!LoadContent(File))
	{
		esyslog("%s: Can't load Menu from '%s'\n",PLUGIN_NAME_I18N, File);
		SetContent(cString::sprintf("%s|||||%s\n%s\n%s\n%s\n%s\n%s\n%s",
								tr(EM[0]), tr(EM[1]), (const char*)cString::sprintf(tr(EM[2]), File), tr(EM[3]), tr(EM[4]), tr(EM[5]), tr(EM[6]), tr(EM[7])
						), /*i18n=*/false , /*fromLoad=*/true );
/*		SetTitle(tr("Error while loading"));
		SetContent(cString::sprintf(tr( "ERROR while loading|||||ERROR while loading\n"
							"File:%s\n"
							"Press 'Yellow' to refresh the data.\n"
							"Check your systemlog for errormessages.\n"
							"Turn on 'Autoupdate' (not recommened).\n"
							"Try other paths with commandline flags.\n"
							"If nothing helps, get an internetconnection ;)."
						), File), cCharSetConv::SystemCharacterTable());
*/
		Skins.Message(mtError, tr("Error while loading"), 2);
	}
}

//##################################################################

void cSportNGMenuText::SetContent(const char *Text, bool i18n/*=true*/, bool fromLoad/*=false*/)
{
	ClearHelpKeys();
	SetHelpKeyAndCommand(2, tr("Refresh"),"r");	// HelpKeys Std
	SetHelpKeyAndCommand(3, tr("Info"), "i");
#if VDRVERSNUM >= 10503
	cCharSetConv myCharSetConv("ISO-8859-15", cCharSetConv::SystemCharacterTable());
	#define CharConv(s) (char*)(i18n?myCharSetConv.Convert(s):s)
#else
	#define CharConv(s) s
#endif
	if(Text)
	{
		char *datas[6];
		SplitString(CharConv(Text), '|', datas, sizeof(datas)/sizeof(datas[0]));

		SetTitle(datas[0]);
		for(int i=0;i<4;i++)	SetHelpKeyAndCommand(i, datas[i+1]);
		SetText(datas[5]);
	}
	DisplayHelpKeys();
}

//##################################################################

void cSportNGMenuText::DisplayHelpKeys()
{
	char *currenthelpKeys[4];
	GetHelpKeys(currenthelpKeys);
	SetHelp(currenthelpKeys[0],currenthelpKeys[1],currenthelpKeys[2],currenthelpKeys[3]);
}

//##################################################################

eOSState cSportNGMenuText::ProcessKey(eKeys Key)
{
	eOSState state=osUnknown;
	bool hadSubmenu = HasSubMenu();

	switch (Key)
	{
	case kRed:
	case kGreen:
	case kYellow:
	case kBlue:
		{
			state = cOsdMenu::ProcessKey(Key);
		
			if(hadSubmenu && !HasSubMenu())
				return osContinue;
		
			if(state==osUnknown)
			{
				state = osContinue;
				switch (Key)
				{
				case kRed:
					return ParseCommand(GetHelpCommand(0));
					break;
				case kGreen:
					return ParseCommand(GetHelpCommand(1));
					break;
				case kYellow:
					return ParseCommand(GetHelpCommand(2));
					break;
				case kBlue:
					return ParseCommand(GetHelpCommand(3));
					break;
				default:
					break;
				}
			}
		}
		break;
	default:
		state = cMenuText::ProcessKey(Key);
		if(hadSubmenu && !HasSubMenu())
		{
			if(GetFileName()) LoadContent(GetFileName());
			return osContinue;
		}
	}
	return state;
}


/* ########################################################
   ------------------- Men Info --------------------------
   ######################################################## */

class cSportNGInfo : public cSportNGMenuText
{
public:
	cSportNGInfo() : cSportNGMenuText(NULL)
	{
		SetTitle(tr("SportNG Info"));
//		SetHelpKeyAndCommand(3, tr("Back"), "b");
/*		SetCols(6);
		Add(new cSportNGItem(cString::sprintf("SportNG v%s by Armin Diedering", SportNG_VERSION)));
		Add(new cSportNGItem("--------------------------------------------------------------------"));
		Add(new cSportNGItem("Email:\t sportng@ardisoft.de"));
		Add(new cSportNGItem("WWW:\t sportng.ardisoft.de"));
		Add(new cSportNGItem("\t sportng.ardisoft.de/vdr-sportng.tgz"));
*/
extern const char	SportNGInfo[];
SetContent(SportNGInfo);
//		Display();
//		DisplayHelpKeys();
	}
};


/* ########################################################
   ------------------- Men Refresh------------------------
   ######################################################## */

cSportNGRefresh::cSportNGRefresh(int Mode, const void* Var) : 
cOsdMenu("")
{
	if(Mode == REFRESH)
	{
		SetTitle(tr("Refresh Data"));
		if(Reload() == osBack)
			cRemote::Put(kBack, true);
	}
	else if(Mode == UPDATE)
	{
		SetTitle(cString::sprintf(tr("Update to %s"), (const char*)Var));
		if(Update((const char*)Var) == osBack)
			cRemote::Put(kBack, true);
	}
	
}
eOSState cSportNGRefresh::Reload()
{
	eOSState State = osContinue;
	Clear();
//	Skins.Flush();
	SetCols(30);
	Display();
	Skins.Flush();
	if(ExecCommand(tr("Checking Network"), cString::sprintf("/bin/sh '%s/%s' network", ScriptDir, "sportng.sh"), /*bool NiceLine=*/false))
		if(ExecCommand(tr("Downloading Data"), cString::sprintf("/bin/sh '%s/%s' download '%s' 'sportng=%s'", ScriptDir, "sportng.sh", DataDir, SportNG_VERSION)))
			if(ExecCommand(tr("Extract Data"), cString::sprintf("/bin/sh '%s/%s' extract '%s'", ScriptDir, "sportng.sh", DataDir)))
				State = osBack;
	if(State == osBack)
	{
		Completed(tr("R E F R E S H   C O M P L E T E D"));
		cCondWait::SleepMs(800);
	}
	else
	{
		SetHelp(NULL,NULL,tr("Refresh"),tr("Back"));
	}
	return State;
}
eOSState cSportNGRefresh::Update(const char* File)
{
	eOSState State = osContinue;
	Clear();
	SetCols(30);
	Display();
	Skins.Flush();
	if(ExecCommand(tr("Checking Network"), cString::sprintf("/bin/sh '%s/%s' network", ScriptDir, "sportng.sh"), /*bool NiceLine=*/false))
		if(ExecCommand(tr("Downloading Data"), cString::sprintf("/bin/sh '%s/%s' update '%s'", ScriptDir, "sportng.sh", File)))
			State = osBack;
	if(State == osBack)
	{
		Completed(tr("D O W N L O A D   C O M P L E T E D"));
		cCondWait::SleepMs(800);
	}
	else
	{
		SetHelp(NULL,NULL,NULL,tr("Back"));
	}
	return State;
}
void cSportNGRefresh::Completed(const char *Message)
{
	Add(new cOsdItem("" , osUnknown, false));
	Add(new cOsdItem("" , osUnknown, false));
	cOsdItem *lastItem = new cOsdItem(Message, osUnknown, false);
	Add(lastItem);
	DisplayItem(lastItem);
	Skins.Flush();
	for(int i=0; i<6; i++)
	{
		cCondWait::SleepMs(75);
		lastItem->SetText("");
		DisplayItem(lastItem);
		Skins.Flush();
		cCondWait::SleepMs(100);
		lastItem->SetText(Message);
		DisplayItem(lastItem);
		Skins.Flush();
	}
}

class MySystem : public cThread
{
private:
	char *cmd;
	int state;
public:
	MySystem(const char *Cmd) : cThread("SportNG")
	{
		cmd = strdup(Cmd);
		state=314159265;
		if(Start());
			while(state==314159265)	cCondWait::SleepMs(3);
	}
	~MySystem() { free(cmd); }
	int State() { return state; }
protected:
	void Action(void)
	{
		state = -1;
		isyslog("%s: system('%s');\n", PLUGIN_NAME_I18N, cmd);
		state = system(cmd);
		isyslog("%s: system('%s')=%i;\n", PLUGIN_NAME_I18N, cmd, state);
	}
};
bool cSportNGRefresh::ExecCommand(const char *Message, const char *Command, bool NiceLine/*=true*/)
{
	cOsdItem *lastItem;
	int res=0;
	if(NiceLine)
	{
		lastItem = new cOsdItem("-------------------------------------------------------------------------------------------------", osUnknown, false);
		Add(lastItem);
		DisplayItem(lastItem);
		Skins.Flush();
	}
	lastItem = new cOsdItem(cString::sprintf("%s\t.....", Message), osUnknown, false);
	Add(lastItem);
	DisplayItem(lastItem);
	Skins.Flush();
	if(Command)
	{
		int i=0;
		int d=1;
		const char *Messages[5] = {"O....",".O...","..O..","...O.","....O"};
		MySystem System(Command);
		while(System.Active())
		{
			lastItem->SetText(cString::sprintf("%s\t%s", Message, Messages[i]));
			DisplayItem(lastItem);
			Skins.Flush();
			i += d;
			if(i==0 || i==4)	d = -d;
			cCondWait::SleepMs(80);
		}
		res = System.State();
//		res = system(Command);
		if(res)
			lastItem->SetText(cString::sprintf("%s\t%s", Message, tr("Failed")));
		else
			lastItem->SetText(cString::sprintf("%s\t%s", Message, tr("OK")));
		DisplayItem(lastItem);
		Skins.Flush();
	}
	return !res;
}
eOSState cSportNGRefresh::ProcessKey(eKeys Key)
{
	bool hadSubmenu = HasSubMenu();
	eOSState state = cOsdMenu::ProcessKey(Key);
	if(hadSubmenu && !HasSubMenu())
		return osContinue;
	if(state==osUnknown)
	{
		state = osContinue;
		switch (Key)
		{
		case kYellow:
			return Reload();
			break;
		case kBlue:
		case kOk:
			return osBack;
			break;
		default:
			;
		}
	}
	return state;
}

